# Tamito KAJIYAMA <18 September 2004>

from Tkinter import *

from Sketch import PolyBezier, CreatePath, Point, ContAngle, ContSmooth
from Sketch import Group, Scale, EmptyLineStyle, SolidPattern, StandardColors
from Sketch.UI.sketchdlg import SKModal
from Sketch.UI.miniscroll import MiniScroller

import os, string, tempfile

import skm17n; skm17n.init()

editor = "medit %s"
params = ("", "", "", 12)

class ParameterDialog(SKModal):
    title = "PolygloText"
    def build_dlg(self):
        self.text = StringVar(self.top)
        self.text.set(params[0])
        self.lang = StringVar(self.top)
        self.lang.set(params[1])
        self.family = StringVar(self.top)
        self.family.set(params[2])
        self.size = DoubleVar(self.top)
        self.size.set(params[3])
        frame = Frame(self.top)
        frame.pack(fill=X)
        label = Label(frame, text="Text", width=6)
        label.pack(side=LEFT)
        entry = Entry(frame, textvariable=self.text)
        entry.pack(side=LEFT)
        button = Button(frame, text="Edit", command=self.edit)
        button.pack(side=LEFT)
        frame = Frame(self.top)
        frame.pack(fill=X)
        label = Label(frame, text="Lang", width=6)
        label.pack(side=LEFT)
        entry = Entry(frame, textvariable=self.lang, width=6)
        entry.pack(side=LEFT)
        frame = Frame(self.top)
        frame.pack(fill=X)
        label = Label(frame, text="Family", width=6)
        label.pack(side=LEFT)
        entry = Entry(frame, textvariable=self.family)
        entry.pack(side=LEFT)
        frame = Frame(self.top)
        frame.pack(fill=X)
        label = Label(frame, text="Size", width=6)
        label.pack(side=LEFT)
        entry = Entry(frame, textvariable=self.size, width=6, justify=RIGHT)
        entry.pack(side=LEFT)
        scroll = MiniScroller(frame, variable=self.size, min=0)
        scroll.pack(side=LEFT, fill=Y)
        frame = Frame(self.top)
        frame.pack(side=BOTTOM, fill=X)
	button = Button(frame, text=" OK ", command=self.ok)
	button.pack(side=LEFT)
	button = Button(frame, text="Cancel", command=self.cancel)
	button.pack(side=RIGHT)
    def edit(self):
        filename = tempfile.mktemp()
        file = open(filename, "w")
        file.write(self.text.get())
        file.close()
        if string.find(editor, "%s") < 0:
            command = editor + " %s"
        else:
            command = editor
        os.system(command % filename)
        file = open(filename)
        self.text.set(file.read())
        file.close()
        os.unlink(filename)
    def get_parameters(self, flag=None):
        text = self.text.get()
        lang = self.lang.get()
        family = self.family.get()
        size = self.size.get()
        if flag is not None:
            global params
            params = (text, lang, family, size)
        return text, string.lower(lang), string.lower(family), size
    def ok(self, *args):
	self.close_dlg(self.get_parameters(1))

def main(context):
    args = ParameterDialog(context.application.root).RunDialog()
    if args is None:
        return
    text, lang, family, size = args
    glyphs = skm17n.text_path(text, lang, family)
    polygons = []
    for contours in glyphs:
        if not contours:
            continue
        paths = []
        for points in contours:
            path = CreatePath()
            i, num_points = 0, len(points)
            x, y, onpoint = points[0]
            last_point = Point(x, y)
            while i <= num_points:
                if i == num_points:
                    x, y, onpoint = points[0]
                else:
                    x, y, onpoint = points[i]
                point = Point(x, y)
                i = i + 1
                if onpoint:
                    path.AppendLine(point)
                    last_point = point
                else:
                    c1 = last_point + (point - last_point) * 2.0 / 3.0
                    x, y, onpoint = points[i % num_points]
                    if onpoint:
                        i = i + 1
                        cont = ContAngle
                    else:
                        x = point.x + (x - point.x) * 0.5
                        y = point.y + (y - point.y) * 0.5
                        cont = ContSmooth
                    last_point = Point(x, y)
                    c2 = last_point + (point - last_point) * 2.0 / 3.0
                    path.AppendBezier(c1, c2, last_point, cont)
            path.ClosePath()
            path.Transform(Scale(size/1024.0))
            paths.append(path)
        polygon = PolyBezier(tuple(paths))
        polygon.AddStyle(EmptyLineStyle)
        polygon.SetProperties(fill_pattern=SolidPattern(StandardColors.black))
        polygons.append(polygon)
    context.main_window.PlaceObject(Group(polygons))

import Sketch.Scripting
Sketch.Scripting.AddFunction('PolygloText', 'PolygloText', main,
                             script_type=Sketch.Scripting.AdvancedScript)
